package com.example.testmali;

import java.nio.Buffer;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.ShortBuffer;
import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;
import android.content.Context;
import android.opengl.GLSurfaceView;
import android.opengl.GLES20;
import android.opengl.Matrix;
import android.util.Log;

public class GLSurface extends GLSurfaceView implements GLSurfaceView.Renderer
{
	private static final String vs_source = 
		"attribute vec4 position;\n" +
		"uniform mat4 mvp;\n" +
		"void main()\n" +
		"{\n" +
			"gl_Position = mvp*position;\n" +
		"}";
	
	private static final String fs_source = 
		"precision mediump float;\n" +
		"void main()\n" +
		"{\n" +
			"gl_FragColor = vec4(1.0);\n" +
		"}";
	
	private int program = 0;
	private int loc_position = 0;
	private int loc_matrix = 0;
	private float[] matrix = new float[16];
	private Buffer vbuffer = null;
	private Buffer ibuffer = null;
	private int vbuffer_id = 0;
	private int ibuffer_id = 0;
	private final int VCOUNT = 8192;
	private final int ICOUNT = 600;
		
	/**
	 * Create object GLSurface
	 */
	public GLSurface(Context context)
	{
        super(context);
    
        // enable touch
        setFocusable(true);
  
        // standtart method to enable GLES 2.0
        this.setEGLContextClientVersion(2);
        this.setEGLConfigChooser(true);
        
        
        // set renderer
		setRenderer(this);
		setRenderMode(RENDERMODE_CONTINUOUSLY);	
	}
	
	/**
	 * create program from sources
	 */
	public int createProgram(String vsource, String fsource)
	{
		// create vertex shader
		int vshader = GLES20.glCreateShader(GLES20.GL_VERTEX_SHADER);
		GLES20.glShaderSource(vshader, vsource);
        GLES20.glCompileShader(vshader);
        
		// create fragment shader
		int fshader = GLES20.glCreateShader(GLES20.GL_FRAGMENT_SHADER);
		GLES20.glShaderSource(fshader, fsource);
        GLES20.glCompileShader(fshader);
        
        // create program
        int program = GLES20.glCreateProgram();
        GLES20.glAttachShader(program, vshader);
        GLES20.glAttachShader(program, fshader);
        GLES20.glLinkProgram(program);
        
        return program;
	}
	
	/**
	 * create buffer object from float-data 
	 */
	public Buffer createBuffer(float[] data)
	{
        FloatBuffer buffer = ByteBuffer.allocateDirect(data.length*4).order(ByteOrder.nativeOrder()).asFloatBuffer();
        buffer.put(data).position(0);
        return buffer;
	}
	
	/**
	 * create buffer object from short-data 
	 */
	public Buffer createBuffer(short[] data)
	{
        ShortBuffer buffer = ByteBuffer.allocateDirect(data.length*2).order(ByteOrder.nativeOrder()).asShortBuffer();
        buffer.put(data).position(0);
        return buffer;
	}
	
	/**
	 * Event of create gl-surface
	 */
	public void onSurfaceCreated(GL10 gl, EGLConfig config)
	{
		GLES20.glDisable(GLES20.GL_DEPTH_TEST);
		GLES20.glDisable(GLES20.GL_CULL_FACE);
		
		Matrix.setIdentityM(matrix, 0);
		
		// create program
		program = createProgram(vs_source, fs_source);
		loc_position = GLES20.glGetAttribLocation(program, "position");
		loc_matrix = GLES20.glGetUniformLocation(program, "mvp");

		// generate buffers
		int ids[] = new int[2];
		GLES20.glGenBuffers(2, ids, 0);
		vbuffer_id = ids[0];
		ibuffer_id = ids[1];
		Log.i("DBG", "vbuffer id "+vbuffer_id);
		Log.i("DBG", "ibuffer id "+ibuffer_id);
		
		// create vertex buffer
		float vdata[] = new float[3*VCOUNT];
		for(int i = 0; i < vdata.length; i++) vdata[i] = (float)(1.0 - 2.0*Math.random());
		vbuffer = createBuffer(vdata);
		GLES20.glBindBuffer(GLES20.GL_ARRAY_BUFFER, vbuffer_id);
		GLES20.glBufferData(GLES20.GL_ARRAY_BUFFER, 4*vbuffer.capacity(), vbuffer, GLES20.GL_STATIC_DRAW);
		
		// create index buffer
		short idata[] = new short[ICOUNT];
		for(int i = 0; i < idata.length; i++) idata[i] = (short)(VCOUNT*Math.random());
		ibuffer = createBuffer(idata);
		GLES20.glBindBuffer(GLES20.GL_ELEMENT_ARRAY_BUFFER, ibuffer_id);
		GLES20.glBufferData(GLES20.GL_ELEMENT_ARRAY_BUFFER, 2*ibuffer.capacity(), ibuffer, GLES20.GL_STATIC_DRAW);
	}

	/**
	 * Event of change gl-surface state
	 */
	public void onSurfaceChanged(GL10 gl, int width, int height)
	{
		GLES20.glViewport(0, 0, width, height);
	}
	
	/**
	 * Event for draw frame
	 */
	private final boolean drawFull = false;
	private int i_offset = 0;
	public void onDrawFrame(GL10 gl)
	{
		GLES20.glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
		GLES20.glClear(GLES20.GL_COLOR_BUFFER_BIT);
		
		GLES20.glBindBuffer(GLES20.GL_ARRAY_BUFFER, vbuffer_id);
		GLES20.glBindBuffer(GLES20.GL_ELEMENT_ARRAY_BUFFER, ibuffer_id);

		GLES20.glUseProgram(program);
		
		GLES20.glUniformMatrix4fv(loc_matrix, 1, false, matrix, 0);
		
		GLES20.glVertexAttribPointer(loc_position, 3, GLES20.GL_FLOAT, false, 0, 0);
		GLES20.glEnableVertexAttribArray(loc_position);
		
		if(drawFull)
		{
			GLES20.glDrawElements(GLES20.GL_TRIANGLES, ICOUNT, GLES20.GL_UNSIGNED_SHORT, 0);
		}else{
			Log.i("DBG", "i_offset "+i_offset);
			GLES20.glDrawElements(GLES20.GL_TRIANGLES, 3, GLES20.GL_UNSIGNED_SHORT, i_offset*2);
			i_offset += 3;
			if((i_offset + 3) > ICOUNT) i_offset = 0;
		}
		
		GLES20.glDisableVertexAttribArray(loc_position);
	}
}
